/**
 * EverythingDopus (c) 2023 Felipe Guedes da Silveira
 *
 * This is a script for Directory Opus.
 * See https://www.gpsoft.com.au/DScripts/redirect.asp?page=scripts for development information.
 *
 * This is the script interface between Directory Opus and EverythingDopusCLI
 *
 * Acknowledgements:
 * This script uses getResourcePathFromGlobalVar() from ExternalCompare.js by
 * Wowbagger. It is used to simplify the setup process, allowing the user to
 * easily choose the location of EverythingDopusCLI ed.exe executable.
 */


//-- Add string trim() method

String.prototype.trim = function(s){s=s||"\\s";return this.replace(new RegExp("^("+s+"){1,}\|("+s+"){1,}$","g"),"");}

// Called by Directory Opus to initialize the script
function OnInit(initData)
{
	initData.name = "EverythingDopus";
	initData.version = "1.2";
	initData.copyright = "(c) 2023 Felipe Guedes da Silveira";
	initData.url = "https://github.com/TheZoc/EverythingDopusCLI";
	initData.desc = "";
	initData.default_enable = true;
	initData.min_version = "12.0";

	// variable for enabling debug, so as to enable debug via a button
	initData.config_desc = DOpus.NewMap();
	initData.config.DEBUG = false;
	initData.config_desc("DEBUG") = "Set DEBUG flag to true in order to enable logging messages to the Opus Output Window";

	initData.config.DebugEnableVar = "$glob:debug";
	initData.config_desc("DebugEnableVar") = "Global var for debug override. Used to enable and disable debug for script applications.";

	var cmd = initData.AddCommand();
	cmd.name = "EverythingDopus";
	cmd.method = "OnEverythingDopusSearch";
	cmd.desc = "Search given string on Everything and add results to a Directory Opus collection";
	cmd.label = "Everything Dopus Search";
	cmd.template = "SearchString/R";

	var cmd2 = initData.AddCommand();
	cmd2.name = "ev";
	cmd2.method = "OnEverythingSearch";
	cmd2.desc = "Create an Opus collection using ES (Everything Search) output";

	var cmd3 = initData.AddCommand();
	cmd3.name = "ev2";
	cmd3.method = "OnEverythingSearch2";
	cmd3.desc = "Create an Opus collection using ES (Everything Search) output (2 buttons)";

	var cmd4 = initData.AddCommand();
	cmd4.name = "ecf";
	cmd4.method = "OnEverythingCurrentFolder";
	cmd4.desc = "Create an Opus collection using ES (Everything Search) output (Current folder)";

	var cmd5 = initData.AddCommand();
	cmd5.name = "eclip";
	cmd5.method = "OnEverythingClipboard";
	cmd5.desc = "Create an Opus collection using ES (Everything Search) output";

	var cmd6 = initData.AddCommand();
	cmd6.name = "fb-tl";
	cmd6.method = "Onfbtl";
	cmd6.desc = "Find by Tags List";
}


// Implement the ExternalCompare command
function OnEverythingDopusSearch(scriptCmdData)
{
	var funcData = scriptCmdData.func;
	LogMessage("OnEverythingDopusSearch - SearchString: " + funcData.args.SearchString);
	var searchString = funcData.args.got_arg.SearchString ? funcData.args.SearchString : "";
	var qualifiers = funcData.qualifiers;
	var fromKeyboard = funcData.fromkey;
	var forceRequestForExternalTool = (!fromKeyboard && (qualifiers.indexOf("shift") > -1));
	LogMessage("OnEverythingDopusSearch - fromKeyboard: " + fromKeyboard + " - forceRequestForExternalTool:" + forceRequestForExternalTool);

	//Get the exe path for the command line search application
	var doFsu = DOpus.FSUtil;
	var exePath = getResourcePathFromGlobalVar("EverythingDopusCLI", "ed.exe", "EverythingDopusCLI", forceRequestForExternalTool);
	if (!exePath || !doFsu.Exists(exePath))
	{
		RaiseError(funcData,"The executable 'ed.exe' for application EverythingDopusCLI could not be found. Try running the script again.");
		return;
	}

	var commandLine = exePath + " " + searchString;
	LogMessage("OnEverythingDopusSearch - command line:" + commandLine);
	funcData.command.runcommand(commandLine);
}

var doLogCmd = DOpus.NewCommand;
function LogMessage(message)
{
  if (Script.config.DEBUG || doLogCmd.IsSet(Script.config.DebugEnableVar)) { DOpus.OutputString(message)};
}

/*
Requested from the user via a file dialog, the path to an executable. The path is then stored in a dopus persistent variable.
Making it accessible next run. Caching pattern.
v1.0

name: name of the application, used in the dialog to assist user.
filename: name of requested file. use in dialog.
varKey: dopus variable key, where the exe path is stored.
forceUserRequest: Force user to find path even if the path exists in the variable. Useful if the user needs to reset. Pass if true shift is down.
*/
function getResourcePathFromGlobalVar(name, filename, varKey, forceUserRequest)
{
	var doFsu = DOpus.FSUtil;
	var doCmd = DOpus.NewCommand;
	var doVars = DOpus.vars;

	var requestReason;

	var resourcePath;

	if(forceUserRequest)
	{
		requestReason = "forced";
	}
	else if(!doVars.Exists(varKey))
	{
		requestReason = "key'" + varKey + "' not found";
	}
	else
	{
		resourcePath = doFsu.Resolve(doVars.Get(varKey));
		if (!doFsu.Exists(resourcePath))
		{
			requestReason = "Path not found:" + resourcePath;
		}
	}

	if(!resourcePath || !doFsu.Exists(resourcePath))
	{
		var dlg = DOpus.Dlg;
		var dlgResult = dlg.Open("Please locate the executable '" +filename + "' for application " + name + " ["+requestReason+"]", doFsu.Resolve("/homeroot"), DOpus.Listers(0));
		if(dlgResult && dlgResult.result)
		{
			resourcePath = doFsu.Resolve(dlgResult);
			if(resourcePath || doFsu.Exists(resourcePath))
			{
				doVars(varKey) = resourcePath;
				doVars(varKey).persist = true;
			}
			else
			{
				var dlg = DOpus.Dlg;
				dlg.title = name + " not found";
				dlg.message = "Can't find file\n Path:" + resourcePath ;
				dlg.buttons = "OK";
				dlg.icon = "error";
				dlg.Show();
				doVars(varKey).Delete();
				resourcePath = null;
			}
		}
		else
		{
			doVars(varKey).Delete();
			resourcePath = null;
		}
	}

	LogMessage("OnEverythingDopusSearch - Path for " +name + " [key: " + varKey + "] = " + resourcePath);

	return resourcePath;
}






/*
===============================================================================
ev
===============================================================================
*/

// Read the history of the EverythingForDopus_History.ini file
  var filepath = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\EverythingForDopus_History.ini";   //File path

function OnEverythingSearch(scriptCmdData, ev)
{
	searchString2 = "";
    var funcData = scriptCmdData.func;
	var path = String(funcData.sourcetab.path);
    fsu = DOpus.FSUtil;
	var ST = DOpus.Create.StringTools();
    if(fsu.Exists(filepath)) {
       var bom = fsu.OpenFile(filepath).Read();
  
       textbom = ST.Decode(bom, "utf-8 bom");
    } else {
       var textbom = "";
    }
    
    var dlg = DOpus.Dlg;
    
    dlg.title = "Directory Opus";                                           // Tittle
    dlg.template = "dlgCombox";
	dlg.detach = true;
    dlg.Create();
	dlg.Control("static").style = "b";
	dlg.Control("static").label = "[Everything]";                           // Topic
	dlg.Control("static2").label = "Enter search term(s)";                  // Notes
	if(!ev) {                                                               // Icon
	   dlg.Control("static3").label = Script.LoadImage("Everything24.ico");
	} else {
	   dlg.Control("static3").label = Script.LoadImage("EverythingDopusCLI-22x22.png");
	   dlg.Control("btn3").enabled = false;   // Make the buttons invisible
	   dlg.Control("btn3").visible = false;
	   dlg.Control("btn4").enabled = false;
	   dlg.Control("btn4").visible = false;
	}
	
    var myArray = textbom.split("\r\n");            //Seprator
	
	if(funcData.sourcetab.selected.count==1){
		promptstr = maybe_quote(funcData.sourcetab.selected(0).name_stem_m);
	   dlg.Control("combo").label = promptstr;      //Default value
	  } else {
	  dlg.Control("combo").label = myArray[0];
	  }
	
	dlg.Control("combo").SelectRange(0, -1);        //Select Default value
    for(i = 0; myArray[i]; i++) {
		dlg.Control("combo").AddItem(myArray[i]);   //Add each item to drop-down list
	    }
    
    dlg.Show();

//msg loop
	do
	{
		msg = dlg.GetMsg();
/*
        //Selection
		if(msg == true && dlg.Control("combo").focus == true) {
		   DOpus.Output(dlg.Control("combo").label);
		}
*/
        //Result
		if(dlg.result == 1) {
		   var fixstr = dlg.Control("combo").label;
		   searchString = fixstr.trim().replace(/\|/g,' |');
		} else {
		if(dlg.result == 0) {
		   var searchString = "";
		   return "";
		  }
		}
		
		//If the button 3 is pressed
		if(dlg.result == 2) {
		   var fixstr = dlg.Control("combo").label;
		   fixstr = fixstr.trim().replace(/\|/g,' |');
		   searchString2 = fixstr;
		   searchString = maybe_quote(path) + " " + fixstr;
		}
		
		//If the button 4 is pressed
		if(dlg.result == 3) {
		   var fixstr = dlg.Control("combo").label;
		   fixstr = fixstr.trim().replace(/\|/g,' |');
		   searchString2 = fixstr;
		   searchString = "parent:" + maybe_quote(path) + " " + fixstr;
		   DOpus.Output(searchString)
		}
	
	} while (msg == true);
    
	var exePath = GetExePath(scriptCmdData);

    if(ev == "CurrentFolder") {
	   searchString2 = searchString
	   searchString = maybe_quote(path) + " " + searchString;
	   DOpus.Output(searchString)
	}

	var commandLine = exePath + " " + searchString;
	LogMessage("OnEverythingDopusSearch - command line:" + commandLine);
	funcData.command.runcommand(commandLine);

// Create a file to save the history
  if(searchString2) {searchString = searchString2}
  if(searchString) {
	var newHistory = "";
    for(i = 0; myArray[i]; i++) {
	    if(i == 15) {break};
        if(searchString != myArray[i]) {newHistory += "\r\n" + myArray[i]};
	   }
    newHistory = searchString + newHistory;
    
    var oFileWrite = fsu.OpenFile (filepath, "wa");
    var utf8blob = ST.Encode(newHistory, "utf-8 bom");
	
    if(oFileWrite.error == 0)
	  {
       oFileWrite.Write (utf8blob)
      }
    oFileWrite.Close ()
    }
}



/*
===============================================================================
ev2
===============================================================================
*/

function OnEverythingSearch2(scriptCmdData)
{
	OnEverythingSearch(scriptCmdData, true)
}



/*
===============================================================================
ecf
===============================================================================
*/

function OnEverythingCurrentFolder(scriptCmdData)
{
	OnEverythingSearch(scriptCmdData, "CurrentFolder")
}



/*
===============================================================================
eclip
===============================================================================
*/

function OnEverythingClipboard(scriptCmdData)
{
    //-- Add string trim() method
    String.prototype.trim = function(s){s=s||"\\s";return this.replace(new RegExp("^("+s+"){1,}\|("+s+"){1,}$","g"),"");}
	
    var cmd = scriptCmdData.func.command;
	
    // Read the history of the EverythingForDopus_History.ini file
    var filepath = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\EverythingForDopus_History.ini";   // File path

    fsu = DOpus.FSUtil;
	var ST = DOpus.Create.StringTools();
    if(fsu.Exists(filepath)) {
       var bom = fsu.OpenFile(filepath).Read();
       textbom = ST.Decode(bom, "utf-8 bom");
    } else {
       var textbom = "";
    }
	
    if (DOpus.GetClipFormat() == 'files')            // Get string from clipboard to default value
    {
	    var clipFiles = DOpus.GetClip('files');
	    var searchString  = clipFiles(0).name_stem_m;
	} else {
	    searchString = DOpus.GetClip();
	}
	
	searchString.trim().replace(/\|/g,' |');
	if(!searchString) {return}
	
	cmd.AddLine("@nofilenamequoting");
	cmd.AddLine('EverythingDopus "' + searchString+ '"')
	cmd.Run();

// Create a file to save the history
	var newHistory = "";
	var myArray = textbom.split("\r\n");            // Seprator
    for(i = 0; myArray[i]; i++) {
	    if(i == 15) {break};
        if(searchString != myArray[i]) {newHistory += "\r\n" + myArray[i]};
	}
    newHistory = searchString + newHistory;
    
    var oFileWrite = fsu.OpenFile (filepath, "wa");
    var utf8blob = ST.Encode(newHistory, "utf-8 bom");
	
    if(oFileWrite.error == 0)
	{
       oFileWrite.Write (utf8blob)
    }
    oFileWrite.Close ()
}






/*
===============================================================================
Find by Tags List
===============================================================================
*/

 var cmd = DOpus.Create.Command();
 // Create folder if does not exist
 if (!DOpus.FSUtil.Exists("%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\")) {
	cmd.Runcommand('CreateFolder "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\"');
 }



function Onfbtl(ScriptCommandData)
{

    var dlg = ScriptCommandData.func.Dlg();
    var tab = ScriptCommandData.func.sourcetab;
    var cmd = ScriptCommandData.func.command;

    if (!DOpus.version.AtLeast("12.30.1"))
    {
	    dlg.request("This script requires 12.30.1 and above!", "OK");
		return
    }

    var strName;

    dlg.title = "Directory Opus";
    dlg.template = "dlgList";
    dlg.detach = true;
    dlg.Create();
	dlg.Control("static").style = "b";
	dlg.Control("static").label = "Find by Tags";                           // Topic
	dlg.Control("static2").label = "Tick the tags that shall be matched via Everyting tool.";   // Notes
	dlg.Control("static3").label = Script.LoadImage("Everything24.ico");    // Icon
	dlg.Control("static4").label = Script.LoadImage("Find.png");            // Icon
	//dlg.Control("static5").label = "D:\\Icons\\Clear.png";                  // Clear

    updateList(dlg, strName);

    dlg.Show();

	var lst = dlg.Control("list");
	var col = lst.columns;



    while (true)
	{
        var msg = dlg.GetMsg();
        if (!msg.result) break;

// Result
        if (msg.event == "focus" || dlg.Control("btnOK").focus == true)
		{
		   var tags = "";
           for (i=0; i < lst.count; i ++)
		   {
                if (lst.GetItemAt(i).checked == 1)
				{
				    tags += "tag:";
					tags += lst.GetItemAt(i).name;
					tags += "|";
                }
           }
		}

// Right click list menu
        if (msg.event == "rclick" && lst.focus == true)
		{

            var dlgMenu = DOpus.Dlg;
            if (lst.value.index > -1)
			{
				dlgMenu.choices = DOpus.Create.Vector("New", "Edit", "Delete", "Clear Filter", "Refresh", "Copy", "Cancel");
				dlgMenu.menu = DOpus.Create.Vector(1, 0, 0, 0, 0, 0, 1);
            } else {
				dlgMenu.choices = DOpus.Create.Vector("OK", "Cancel");
                dlgMenu.menu = DOpus.Create.Vector(1, 1);
            }
            var menuReturn = dlgMenu.Show;

            if (menuReturn == 1) {
				NewTags();
				updateList(dlg, strName);
            }

            if (menuReturn == 2) {
				var editTag = lst.GetItemAt(lst.value.index).name;
				tab.vars.Set('editTag', editTag);
				EditTag();
				updateList(dlg, strName);
            }

            if (menuReturn == 3) {
				var deleteTag = lst.GetItemAt(lst.value.index).name;
				tab.vars.Set('deleteTag', deleteTag);
				DeleteTag();
				updateList(dlg, strName);
            }

            if (menuReturn == 4) {
                dlg.Control("editFilter").value = "";
			    updateList(dlg, strName);
            }

            if (menuReturn == 5) {
				updateList(dlg, strName);
            }

            if (menuReturn == 6) {
                var item = lst.GetItemAt(lst.value.index).name;
				DOpus.SetClip(item);
            }
        }

// Filter
		if (msg.event == "editchange" || msg.event == "click")
		{
			updateList(dlg, strName);
		}

    }


	if (tags != "" && dlg.result == 1)
	{
	    tags = tags.substr(0, tags.length-1);

	    var argstring = tags.replace(/\|/g,'^|');
	    argstring = argstring.replace(/\</g,'^<');
	    searchString = argstring.replace(/\>/g,'^>');
	    
	    var exePath = GetExePath(ScriptCommandData)
		
	    var commandLine = exePath + " " + searchString;
	    LogMessage("OnEverythingDopusSearch - command line:" + commandLine);
	    cmd.runcommand(commandLine);
	}
}






/*
===============================================================================
Functions
===============================================================================
*/

	var fsu = DOpus.FSUtil;
	var ST = DOpus.Create.StringTools();

function GetExePath(scriptCmdData) {
	var funcData = scriptCmdData.func;
	var qualifiers = funcData.qualifiers;
	var fromKeyboard = funcData.fromkey;
	var forceRequestForExternalTool = (!fromKeyboard && (qualifiers.indexOf("shift") > -1));
	LogMessage("OnEverythingDopusSearch - fromKeyboard: " + fromKeyboard + " - forceRequestForExternalTool:" + forceRequestForExternalTool);
	
	//Get the exe path for the command line search application
	var doFsu = DOpus.FSUtil;
	var exePath = getResourcePathFromGlobalVar("EverythingDopusCLI", "ed.exe", "EverythingDopusCLI", forceRequestForExternalTool);
	if (!exePath || !doFsu.Exists(exePath))
	{
		RaiseError(funcData,"The executable 'ed.exe' for application EverythingDopusCLI could not be found. Try running the script again.");
		return;
	}
	return exePath
}



function maybe_quote(str){
	// Avoid unnecessary use of quotes and always force to string type
	return (String(str).indexOf(" ")<0) ? String(str) : '"'+String(str)+'"';
}



 var tab = DOpus.listers.lastactive.activetab;



function updateList(dlg, strName) {

    var lst = dlg.Control("list");
	var col = lst.columns;

// Add columns
	var vecOrder = col.GetDisplayOrder();
    if (vecOrder.size < 1)
	{
	    col.AddColumn("0:Name");
	    col.AddColumn("1:Index");
	}

    lst.redraw = false;
    lst.RemoveItem(-1);

// List items
    //var text = "tag1, tag2, tag3, tag4, tag5, tag6, tag7, tag8, tag9, tag10, tag11, tag12, tag13, tag14, tag15, tag16, tag17, tag18, tag19, tag20, tag21, tag22, tag23, tag24, tag25, tag26, tag27, tag28, tag29, tag30, tag31, tag32, tag33, tag34, tag35";
	Checktag();
	var text = tab.vars.Get('DOpusTag');
	var myArray = text.split(", ");                           // Seprator

	var idx;
	var count = 0;
	var ii = 0;

    // Get var ADScheck
    var ADS = tab.vars.Get('ADScheck');

// Add checked list
    if (ADS != "")
	{
	    var array = ADS.split(", ");                          // Seprator
		for (i = 0; array[i]; i++)
		{
		     count++;
			 var checkedTag = array[i];

	         var strFilter = dlg.Control("editFilter").value;   // Get filter value
	         strFilter = strFilter.toLowerCase();

		     if (checkedTag.search(strFilter) == -1) {
			     continue;
		     }
	         idx = lst.AddItem(array[i]);                     // Add each item to list
             lst.GetItemAt(idx).subitems(0) = count;          // Add number to column 1 (Index)
			 lst.GetItemAt(idx).checked = 1;

             ii++;
			 var checkedCase = 1;
			 checkedTag = "";
			 strFilter = "";
	    }
    }

// Add list
    var tagCase;
    for (i = 0; myArray[i]; i++)
	{
        strName = myArray[i];
		var strRealName = myArray[i];

        // Remove Duplicate
        tagCase = "";
		if (checkedCase == 1)
		{
			for (ii = 0; array[ii]; ii++)
			{
				if (array[ii] == strName)
				{
					tagCase	= 1;
				}
			}
		}

		if (tagCase == "")
		{
			count++;
	        var strFilter = dlg.Control("editFilter").value;      // Get filter value
	        strFilter = strFilter.toLowerCase();

		    if (strRealName.search(strFilter) == -1) {
			    continue;
		    }

            idx = lst.AddItem(strName);                           // Add each item to list
		    lst.GetItemAt(idx).subitems(0) = count;               // Add number to column 1 (Index)

            ii++;
		    strRealName = "";
		    strFilter = "";
		}
    }

// Set the columns display order
	if (true)
	{
	    col.SetDisplayOrder(1,0);
	}

// Resize columns based on number of lists
    //col.AutoSize();
	col.GetColumnAt(1).width = String(i).length+25;
	if (ii > 27)
	{
	   col.GetColumnAt(0).width = 420-String(i).length;
	}
	else
	{
	   col.GetColumnAt(0).width = 437-String(i).length;
	}
	lst.redraw = true;
}



function Checktag()
{

    var eItem, ADSfile, text, textbom, myArray, ADS2, ADS = "", i = 0;

	if (tab.selected.count > 0)
	{
        for (var eSel = new Enumerator(tab.selected); !eSel.atEnd(); eSel.moveNext())
        {
	         i++;
	         ADS2 = ADS;
	         eItem = eSel.item();
             ADSfile =  eItem + ":Tags";
             // If ADSfile exists
             if (fsu.Exists(ADSfile))
             {
                 text = fsu.OpenFile(ADSfile).Read();          // Read file
                 textbom = ST.Decode(text, "utf-8 bom");       // Decode as UTF-8 with BOM
                 myArray = textbom.split("\r\n");              // The seprator is \r\n
                 ADS = myArray[0];                             // Get the first line
             }
             else {ADS = ""; break}
	         if (i > 1 && ADS != ADS2) {ADS = ""; break};
        }
    }
	ADS = format_tags(ADS);
    tab.vars.Set('ADScheck', ADS);

// Read the tag file DOpusTag.dat and set it to variable DOpusTag
    var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
    if (fsu.Exists(tagFile))
    {
        var tags = fsu.OpenFile(tagFile).Read();
        var tags = ST.Decode(tags, "utf-8 bom");
    } else {var tags = "";}
	tags = format_tags(tags);
    tab.vars.Set('DOpusTag', tags);
}



function NewTags() {

    var dlgEdit = DOpus.Dlg;
    dlgEdit.title = "Directory Opus";
    dlgEdit.template = "dlgEditTags";
    dlgEdit.detach = true;
    dlgEdit.Create();

	dlgEdit.Control("static").style = "b";
	dlgEdit.Control("static").label = 'Enter New Tags';                         // Topic
	dlgEdit.Control("static2").label = 'One tag per line, or enter new tags separated by commas.';   // Notes
	dlgEdit.Control("static3").label = Script.LoadImage("Edit-new.png");        // Icon

	dlgEdit.Control("editTags").value = "";                                     // Default value

    dlgEdit.Show();

	var newTags, newTags2 = "", tagsTMP = "|", tagCase, dlgValueOld, dlgValue = "";

// msg loop
	do
	{
	   dlgValueOld = dlgEdit.Control("editTags").value;
	   msg = dlgEdit.GetMsg();

	   if (msg == true) {dlgValue = dlgEdit.Control("editTags").value};

	   if (msg.qualifiers == "none") {tagsTMP = dlgEdit.Control("editTags").value};

	   var dlgValueArray = dlgValue.split("\r");
	   var newDlgValue = "";
	   for (i = 0; dlgValueArray[i]; i++)
	   {
		   newDlgValue += dlgValueArray[i].trim();
	   }

	   var OldArray = dlgValueOld.split("\r");
	   var dlgValueOld2 = "";
	   for (i = 0;OldArray[i]; i++)
	   {
		   dlgValueOld2 += OldArray[i].trim()
	   }

       // Result
	   if (dlgEdit.result == 1 || msg.qualifiers == "ctrl" && msg.event == "editchange" && newDlgValue == dlgValueOld2 && dlgValue != DOpus.GetClip || msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValue.trim() == tagsTMP.trim() || msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValueArray[0].trim() == "" && dlgValue.trim() != "")
	   {
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValueArray[0].trim() == "" && dlgValue.trim() != "")
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("0")
			   newTags = dlgValue
		   }
		   else
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValue.trim() == tagsTMP.trim())
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("1")
			   newTags = dlgValue
		   }
		   else
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && newDlgValue == dlgValueOld2 && dlgValue != DOpus.GetClip)
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("2")
			   newTags = dlgValueOld
		   }
		   else
		   {
			   newTags = dlgEdit.Control("editTags").value
			   //DOpus.Output("3")
		   }

		   if (newTags.trim() == "") {return};
		   newTags = format_tags(newTags);
		   newTags = remove_duplicate(newTags);

		   var tags = tab.vars.Get('DOpusTag');
		   var tagsArray = tags.split(", ");                            // Seprator

           var array = newTags.split(", ");
           // Remove Duplicate
		   for (i = 0; array[i]; i++)
		   {
			    tagCase = "";
		        for (ii = 0; tagsArray[ii]; ii++)
				{
				     if (array[i] == tagsArray[ii])
					 {
						 tagCase = 1;
					 }
				}

			    if (tagCase != 1)
				{
					newTags2 += array[i] + ", ";
				}
           }

           if (newTags2 == "") return;
		   var newTags3 = newTags2 + tags;
           tab.vars.Set('DOpusTag', newTags3);

           // Write new tags to DOpusTag.dat
		   var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
		   var oFileWrite = fsu.OpenFile(tagFile, "wa");
           var utf8bom = ST.Encode(newTags3	, "utf-8 bom");   // Decode as UTF-8 with BOM
           if (oFileWrite.error == 0)
           {
               oFileWrite.Write(utf8bom)
           }
              oFileWrite.Close()
	   }
	} while (msg == true)
}



function EditTag() {

    var dlgEdit = DOpus.Dlg;
    dlgEdit.title = "Directory Opus";
    dlgEdit.template = "dlgEditTags";
    dlgEdit.detach = true;
    dlgEdit.Create();

	dlgEdit.Control("static").style = "b";
	dlgEdit.Control("static").label = 'Edit Tags';                              // Topic
	dlgEdit.Control("static2").label = 'One tag per line, or enter new tags separated by commas.';   // Notes
	dlgEdit.Control("static3").label = Script.LoadImage("Edit.png");            // Icon

	var editTag = tab.vars.Get('editTag', editTag);
	dlgEdit.Control("editTags").value = editTag;                                // Default value

    dlgEdit.Show();

	var newTag, tagsTMP = "|", tagCase, dlgValueOld, dlgValue = "";

// msg loop
	do
	{
	   dlgValueOld = dlgEdit.Control("editTags").value;
	   msg = dlgEdit.GetMsg();

	   if (msg == true) {dlgValue = dlgEdit.Control("editTags").value};

	   if (msg.qualifiers == "none") {tagsTMP = dlgEdit.Control("editTags").value};

	   var dlgValueArray = dlgValue.split("\r");
	   var newDlgValue = "";
	   for (i = 0; dlgValueArray[i]; i++)
	   {
		   newDlgValue += dlgValueArray[i].trim();
	   }

	   var OldArray = dlgValueOld.split("\r");
	   var dlgValueOld2 = "";
	   for (i = 0;OldArray[i]; i++)
	   {
		   dlgValueOld2 += OldArray[i].trim()
	   }

       // Result
	   if (dlgEdit.result == 1 || msg.qualifiers == "ctrl" && msg.event == "editchange" && newDlgValue == dlgValueOld2 && dlgValue != DOpus.GetClip || msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValue.trim() == tagsTMP.trim() || msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValueArray[0].trim() == "" && dlgValue.trim() != "")
	   {
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValueArray[0].trim() == "" && dlgValue.trim() != "")
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("0")
			   newTag = dlgValue
		   }
		   else
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValue.trim() == tagsTMP.trim())
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("1")
			   newTag = dlgValue
		   }
		   else
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && newDlgValue == dlgValueOld2 && dlgValue != DOpus.GetClip)
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("2")
			   newTag = dlgValueOld
		   }
		   else
		   {
			   newTag = dlgEdit.Control("editTags").value
			   //DOpus.Output("3")
		   }

		   if (newTag.trim() == "") {return};
		   newTag = format_tags(newTag);
		   newTag = remove_duplicate(newTag);

		   // Remove duplicate
	       var tags = tab.vars.Get('DOpusTag');
		   var tagsArray = newTag.split(", ");
		   var tagsArray2 = tags.split(", ");                            // Seprator
		   var tagCase, newTags = "";
		   for (i = 0; tagsArray2[i]; i++)
		   {
			    tagCase = "";
			    for (ii = 0; tagsArray[ii]; ii++)
				{
				     if (tagsArray2[i] == tagsArray[ii])
					 {
						 tagCase = 1;
					 }
				}
				if (tagCase != 1 && tagsArray2[i])
				{
					newTags += tagsArray2[i] + ", ";
				}
		   }

           if (newTags == "") return;
		   newTags = newTag + ", " + newTags;
		   newTags = newTags.substr(0, newTags.length-2);
		   tab.vars.Set('DOpusTag', newTags);

		   // Write new tags to DOpusTag.dat
		   var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
		   var oFileWrite = fsu.OpenFile(tagFile, "wa");
           var utf8bom = ST.Encode(newTags, "utf-8 bom");   // Decode as UTF-8 with BOM
           if (oFileWrite.error == 0)
           {
               oFileWrite.Write(utf8bom)
           }
              oFileWrite.Close()
	   }
	} while (msg == true)
}



function DeleteTag() {

    var deleteTag = tab.vars.Get('deleteTag');
	if (deleteTag == "") return;

	var tags = tab.vars.Get('DOpusTag');
	var tagsArray = tags.split(", ");                           // Seprator
	var newTags = "";
	for (i = 0; tagsArray[i]; i++)
	{
	     if (tagsArray[i] != deleteTag)
		 {
			 newTags += tagsArray[i] + ", ";
         }
	}

    newTags = newTags.substr(0, newTags.length-2)
    tab.vars.Set('DOpusTag', newTags);

    // Write new tags to DOpusTag.dat
	var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
	var oFileWrite = fsu.OpenFile(tagFile, "wa");
    var utf8bom = ST.Encode(newTags, "utf-8 bom");   // Decode as UTF-8 with BOM
    if (oFileWrite.error == 0)
    {
        oFileWrite.Write(utf8bom)
    }
        oFileWrite.Close()
}



function format_tags(str){

    var array = String(str).split(",");                                // Seprator
	var newTag1 = array[0].trim().replace(/\|/g,'');
	if (newTag1 == "") return newTags = "";

    var tag, newTags = "";
	for (i = 0; array[i]; i++)
	{
		 tag = array[i].trim().replace(/\|/g,'');
		 if (tag != "")
		 {
		     newTags += tag + ", ";
		 }
    }
	newTags = newTags.substr(0, newTags.length-2);

    var array2 = newTags.split("\r\n");                                // Seprator
	newTags = "";
	for (i = 0; array2[i]; i++)
	{
		 tag = array2[i].trim().replace(/\|/g,'');
		 if (tag != "")
		 {
		     newTags += tag + ", ";
		 }
	}
	newTags = newTags.substr(0, newTags.length-2);
	return newTags;
}



function remove_duplicate(str) {

    var array = String(str).split(", ");                               // Seprator
	var newTag1 = array[0].trim().replace(/\|/g,'');
	if (newTag1 == "") {return newTag1};

    var vecTags = DOpus.NewVector();
    for (i=0; array[i]; i++)
    {
	     vecTags.push_back = array[i];
    }

    var removedNum = vecTags.unique();
    if (removedNum == 0) {return String(str)};

    var newTags = "";
    for (var eItems = new Enumerator(vecTags); !eItems.atEnd(); eItems.moveNext() )
    {
	     newTags += eItems.item() + ", ";
    }
    newTags = newTags.substr(0, newTags.length-2);
	return newTags;
}



==SCRIPT RESOURCES
<resources>
	<resource name="dlgCombox" type="dialog">
		<dialog fontsize="8" height="88" lang="english" resize="yes" title="Directory Opus" width="322">
			<control edit="yes" height="40" name="combo" type="combo" width="306" x="8" y="50" />
			<control close="3" height="14" name="btn4" resize="xy" title="&amp;Here" type="button" width="60" x="81" y="67" />
			<control close="2" height="14" name="btn3" resize="xy" title="&amp;Below" type="button" width="60" x="145" y="67" />
			<control halign="left" height="8" name="static" title="Topic" type="static" valign="top" width="280" x="30" y="10" />
			<control halign="left" height="16" name="static2" title="Notes" type="static" valign="top" width="280" x="30" y="22" />
			<control halign="center" height="14" image="yes" name="static3" title="Icon" type="static" valign="top" width="14" x="8" y="7" />
			<control close="1" default="yes" height="14" name="btnOK" resize="xy" title="&amp;OK" type="button" width="50" x="209" y="67" />
			<control close="0" height="14" name="btnCancel" resize="xy" title="&amp;Cancel" type="button" width="50" x="263" y="67" />
		</dialog>
	</resource>
	<resource name="dlgEditTags" type="dialog">
		<dialog fontsize="8" height="120" lang="english" resize="yes" title="Edit Tags" width="522">
			<control halign="left" height="62" multiline="yes" name="editTags" resize="wh" type="edit" width="506" x="8" y="32" />
			<control close="1" default="yes" height="14" name="btnApply" resize="xy" title="&amp;Apply" type="button" width="50" x="409" y="99" />
			<control close="0" height="14" name="btnCancel" resize="xy" title="&amp;Cancel" type="button" width="50" x="462" y="99" />
			<control halign="left" height="8" name="static" title="Edit Tags" type="static" valign="top" width="472" x="39" y="7" />
			<control halign="left" height="20" image="yes" name="static3" title="Static" type="static" valign="top" width="20" x="10" y="4" />
			<control halign="left" height="8" name="static2" title="Notes" type="static" valign="top" width="472" x="39" y="17" />
		</dialog>
	</resource>
	<resource name="dlgList" type="dialog">
		<dialog fontsize="9" height="350" lang="english" resize="yes" title="Directory Opus" width="279">
			<control halign="left" height="12" name="editFilter" resize="xy" tip="Filter" type="edit" width="118" x="21" y="331" />
			<control checkboxes="auto" fullrow="yes" height="287" name="list" noheader="yes" resize="wh" type="listview" viewmode="details" width="266" x="6" y="40" />
			<control close="1" default="yes" height="12" name="btnOK" resize="xy" title="&amp;OK" type="button" width="42" x="184" y="332" />
			<control close="0" height="12" name="btnClose" resize="xy" title="C&amp;ancel" type="button" width="42" x="229" y="332" />
			<control halign="left" height="8" name="static" title="Topic" type="static" valign="top" width="245" x="25" y="7" />
			<control halign="left" height="13" image="yes" name="static3" title="Static" type="static" valign="top" width="13" x="7" y="7" />
			<control halign="left" height="12" image="yes" name="static4" resize="xy" title="Static" type="static" valign="top" width="12" x="8" y="331" />
			<control halign="left" height="16" name="static2" title="Notes" type="static" valign="top" width="245" x="25" y="20" />
			<control halign="left" height="5" name="static5" title="Static" type="static" valign="top" width="6" x="130" y="335" />
		</dialog>
	</resource>
</resources>